'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var isEmpty = _interopDefault(require('lodash/isEmpty'));
var get = _interopDefault(require('lodash/get'));
var cloneDeep = _interopDefault(require('lodash/cloneDeep'));
var lodash = require('lodash');

// 自定义属性，用于标识当前文本被分割
var customAttr = 'data-wj-custom-text';
var customTag = 'span';
var customSplitAttr = 'data-wj-custom-split';
var customRowKey = 'id';
var customSelectedAttr = 'data-wj-custom-id';
var marginVertical = 15;

/**
 * 获取层级
 * @param node
 * @param TopNode
 */
var getLevelList = function getLevelList(node, TopNode) {
  var list = [];

  while (node !== TopNode) {
    var parent = node.parentNode;
    var children = parent.childNodes;

    for (var i = 0; i < children.length; i++) {
      if (node === children[i]) {
        list.push(i);
        break;
      }
    }

    node = parent;
  }

  return list.reverse();
};

/**
 * @description 如果外层有 customAttr 属性， 表示当前节点为自定义分割节点，取最近的一层, 如果没有就取当前节点
 * @param startContainer
 * @param noteContainer
 */

var getStartNode = function getStartNode(startContainer, noteContainer) {
  var node = startContainer;
  var lastNode = startContainer;

  while (node && node !== noteContainer) {
    var isCustom = node.getAttribute && node.getAttribute(customAttr);

    if (isCustom) {
      lastNode = node;
      break;
    }

    node = node.parentNode;
  }

  return lastNode;
};

/**
 * @description 分割文本节点作为一个整体判断
 */
var getCustomSplitNodeInfo = function getCustomSplitNodeInfo(_ref) {
  var node = _ref.node,
      _ref$isStart = _ref.isStart,
      isStart = _ref$isStart === void 0 ? false : _ref$isStart,
      range = _ref.range;
  var endContainer = range.endContainer,
      endOffset = range.endOffset,
      startContainer = range.startContainer,
      startOffset = range.startOffset;
  var info = {
    isStart: false,
    isEnd: false,
    start: 0,
    end: 0
  };

  if (isStart) {
    info.isStart = true;
    info.start = 0;
  } else if (node.contains(startContainer)) {
    info.isStart = true;
    var startNode = startContainer;
    var offset = 0;

    outer: while (startNode && startNode !== node) {
      if (startNode === startContainer) {
        offset += startOffset;
      } else {
        var _startNode, _startNode$textConten;

        offset += ((_startNode = startNode) == null ? void 0 : (_startNode$textConten = _startNode.textContent) == null ? void 0 : _startNode$textConten.length) || 0;
      }

      if (startNode.previousSibling) {
        startNode = startNode.previousSibling;
      } else {
        do {
          startNode = startNode.parentNode;

          if (node === startNode) {
            break outer;
          }
        } while (!startNode.previousSibling);

        startNode = startNode.previousSibling;
      }
    }

    info.start = offset;
  }

  if (node.contains(endContainer)) {
    info.isEnd = true;
    var endNode = endContainer;
    var _offset = 0;

    outer: while (endNode && endNode !== node) {
      if (endNode === endContainer) {
        _offset += endOffset;
      } else {
        var _endNode, _endNode$textContent;

        _offset += ((_endNode = endNode) == null ? void 0 : (_endNode$textContent = _endNode.textContent) == null ? void 0 : _endNode$textContent.length) || 0;
      }

      if (endNode.previousSibling) {
        endNode = endNode.previousSibling;
      } else {
        do {
          endNode = endNode.parentNode;

          if (node === endNode) {
            break outer;
          }
        } while (!endNode.previousSibling);

        endNode = endNode.previousSibling;
      }
    }

    info.end = _offset;
  } else if (info.isStart) {
    var _node$textContent;

    info.isEnd = false;
    info.end = (node == null ? void 0 : (_node$textContent = node.textContent) == null ? void 0 : _node$textContent.length) || 0;
  }

  return info;
};

var defaultCustomNodeInfo = {
  isStart: false,
  isEnd: false,
  start: 0,
  end: 0
};

var getSelectedInfo = function getSelectedInfo(_ref) {
  var range = _ref.range,
      noteContainer = _ref.noteContainer;
  var commonAncestorContainer = range.commonAncestorContainer,
      endContainer = range.endContainer,
      endOffset = range.endOffset,
      startContainer = range.startContainer,
      startOffset = range.startOffset;
  var outerContainer = getStartNode(commonAncestorContainer, noteContainer);
  var topLevel = getLevelList(outerContainer, noteContainer); // TODO 没太明白 Node  和  Element 的区别

  var node = getStartNode(startContainer, noteContainer);
  var isStart = false;
  var isEnd = false;
  var list = [];
  var customNodeInfo = defaultCustomNodeInfo;

  outer: while (node && outerContainer.contains(node) && !isEnd) {
    // 所有分割文本上都多嵌套了一层 span ，添加自定义属性 data-wj-custom-text
    // 文本节点木有获取属性
    var isCustom = node.getAttribute && node.getAttribute(customAttr); // TODO 需要判断当前是否为自定义分割节点，为分割节点的情况，需要到内部去判断当前是否为开始节点与结束节点

    if (isCustom) {
      customNodeInfo = getCustomSplitNodeInfo({
        range: range,
        isStart: isStart,
        node: node
      });
      isStart = customNodeInfo.isStart;
      isEnd = customNodeInfo.isEnd;
    } else {
      if (node === startContainer) {
        isStart = true;
      }

      if (node === endContainer) {
        isEnd = true;
      }
    } // 选中开始节点 - 选中的结束节点 之间的文本节点都为选中文本


    if (isStart) {
      var level = getLevelList(node, outerContainer); // 只标记文本节点

      if (node.nodeType === 3) {
        var _node, _node$textContent, _node$textContent2;

        var textStartOffset = 0;
        var textEndOffset = ((_node = node) == null ? void 0 : (_node$textContent = _node.textContent) == null ? void 0 : _node$textContent.length) || 0;

        if (node === startContainer) {
          textStartOffset = startOffset;
        }

        if (node === endContainer) {
          textEndOffset = endOffset;
        }

        list.push({
          level: [].concat(topLevel, level),
          start: textStartOffset,
          end: textEndOffset,
          text: ((_node$textContent2 = node.textContent) == null ? void 0 : _node$textContent2.slice(textStartOffset, textEndOffset)) || ''
        });
      } else if (isCustom) {
        var _node2, _node2$textContent;

        var _customNodeInfo = customNodeInfo,
            _customNodeInfo$start = _customNodeInfo.start,
            start = _customNodeInfo$start === void 0 ? 0 : _customNodeInfo$start,
            _customNodeInfo$end = _customNodeInfo.end,
            end = _customNodeInfo$end === void 0 ? 0 : _customNodeInfo$end;
        list.push({
          level: [].concat(topLevel, level),
          start: start,
          end: end,
          text: ((_node2 = node) == null ? void 0 : (_node2$textContent = _node2.textContent) == null ? void 0 : _node2$textContent.slice(start, end)) || ''
        }); // TODO 综合当前节点选中文本 ， 待完善
      }
    } // 自定义节点为最底层的文本节点，逻辑上无法分割， 所以转到查询兄弟节点


    if (node.childNodes.length && !isCustom) {
      // TODO 不造类型该咋写，头疼
      node = node.childNodes[0];
    } else if (node.nextSibling) {
      node = node.nextSibling;
    } else {
      do {
        node = node.parentNode;

        if (noteContainer === node) {
          break outer;
        }
      } while (!node.nextSibling);

      node = node.nextSibling;
    }
  }

  return list.filter(function (item) {
    return item.text && item.text.trim();
  });
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var customValue = {};
var getCustomValue = function getCustomValue(key) {
  return key ? customValue[key] : customValue;
};
var setCustomValue = function setCustomValue(value) {
  customValue = _extends({}, customValue, value);
};

var context = /*#__PURE__*/React.createContext(null);

// TODO 后续有性能问题再优化
var resolveIntersection = function resolveIntersection(list, content) {
  var joinList = [];
  var startOffset = 0;
  var textContentList = [];

  var _loop = function _loop(i) {
    list.forEach(function (e) {
      var d = e.d,
          uuid = e.u,
          mode = e.m;
      var start = d.start,
          end = d.end;

      if (start === i) {
        if (startOffset < i) {
          textContentList.push({
            start: startOffset,
            end: i,
            text: content.slice(startOffset, i),
            options: [].concat(joinList)
          });
        }

        startOffset = i;
        joinList.push({
          uuid: uuid,
          mode: mode
        });
      } else if (end === i) {
        var index = joinList.findIndex(function (item) {
          return item.uuid === uuid;
        });

        if (index === -1) {
          return;
        }

        if (startOffset < i) {
          textContentList.push({
            start: startOffset,
            end: i,
            text: content.slice(startOffset, i),
            options: [].concat(joinList)
          });
        }

        startOffset = i;
        joinList.splice(index, 1);
      }
    });
  };

  for (var i = 0; i <= content.length; i++) {
    _loop(i);
  }

  if (startOffset !== content.length) {
    textContentList.push({
      start: startOffset,
      end: content.length,
      text: content.slice(startOffset, content.length),
      options: [].concat(joinList)
    });
  }

  return textContentList;
};

var getLastAst = function getLastAst(astItem, path) {
  if (astItem.isCustom) return astItem;
  return get(astItem, path) || get(astItem.children, path) || astItem;
};

var translateAstNodes = function translateAstNodes(ast, options) {
  // TODO 数据合并
  var _getCustomValue = getCustomValue(),
      _getCustomValue$tagNa = _getCustomValue.tagName,
      tagName = _getCustomValue$tagNa === void 0 ? customTag : _getCustomValue$tagNa,
      _getCustomValue$split = _getCustomValue.splitAttrName,
      splitAttrName = _getCustomValue$split === void 0 ? customSplitAttr : _getCustomValue$split,
      _getCustomValue$attrN = _getCustomValue.attrName,
      attrName = _getCustomValue$attrN === void 0 ? customAttr : _getCustomValue$attrN,
      _getCustomValue$selec = _getCustomValue.selectedAttr,
      selectedAttr = _getCustomValue$selec === void 0 ? customSelectedAttr : _getCustomValue$selec,
      _getCustomValue$rowKe = _getCustomValue.rowKey,
      rowKey = _getCustomValue$rowKe === void 0 ? customRowKey : _getCustomValue$rowKe,
      _getCustomValue$modeC = _getCustomValue.modeClassNames,
      modeClassNames = _getCustomValue$modeC === void 0 ? {} : _getCustomValue$modeC;

  var translateNodeList = [];
  if (!options || isEmpty(options)) return;
  options.forEach(function (optionItem, index) {
    var list = optionItem.list,
        mode = optionItem.mode;
    var uuid = optionItem[rowKey];
    if (!Array.isArray(list) || !list.length) return;
    list.forEach(function (item) {
      // @ts-ignore
      var node = item.level.reduce(getLastAst, ast);
      var type = node.type,
          attributes = node.attributes;

      if (type === 'text' || type === 'element' && attributes.find(function (item) {
        return item.name === attrName;
      })) {
        var parentNode = node.parent;
        if (!parentNode) return;
        var cNode = node;

        if (!node.isCustom) {
          cNode = {
            attributes: [{
              name: attrName,
              value: 'true'
            }],
            isCustom: true,
            custom: {
              list: [{
                d: item,
                m: mode,
                u: uuid
              }],
              node: node
            },
            children: [],
            parent: node.parent,
            tagName: tagName,
            type: 'element'
          };
          translateNodeList.push(cNode);

          var _index = parentNode.children.findIndex(function (item) {
            return item === node;
          });

          if (_index !== -1) {
            parentNode.children.splice(_index, 1, cNode);
          }
        } else {
          cNode.custom.list.push({
            d: item,
            m: mode,
            u: uuid
          });
        }
      }
    });
  });
  translateNodeList.forEach(function (item) {
    var _item$children;

    var _item$custom = item.custom,
        list = _item$custom.list,
        node = _item$custom.node; // 解决字符转义后路径对应问题

    var content = node.content; // 过滤不匹配文本

    var filterData = list.filter(function (item) {
      var _item$d = item.d,
          start = _item$d.start,
          end = _item$d.end,
          text = _item$d.text;
      var comparisonText = content.slice(start, end);
      return comparisonText === text;
    });
    var children = resolveIntersection(filterData, content).map(function (item) {
      var text = item.text,
          options = item.options; // 用于解决文本转标签注入的问题

      var content = text;

      if (isEmpty(options)) {
        return {
          content: content,
          type: 'text'
        };
      } // @ts-ignore


      var ref = {
        children: []
      };
      var lastNode = options.reduce(function (parent, option) {
        var uuid = option.uuid,
            mode = option.mode;
        var item = {
          type: 'element',
          tagName: tagName,
          parent: null,
          attributes: [{
            name: 'class',
            value: mode ? modeClassNames[mode] : ''
          }, {
            name: splitAttrName,
            value: 'true'
          }, {
            name: selectedAttr,
            value: uuid
          }],
          children: []
        };
        parent.children.push(item);
        return item;
      }, ref);
      lastNode.children.push({
        content: content,
        type: 'text'
      });
      return ref.children[0];
    });

    (_item$children = item.children).push.apply(_item$children, children);
  });
};

var getJSON = function getJSON(ast, list) {
  translateAstNodes(ast, list);
  return ast;
};

// 将HTML转义为实体
/**
 * 设置class
 */

function classNames() {
  var _arguments = arguments;
  // @ts-ignore
  var classNameMap = {};

  var _loop = function _loop(i) {
    var name = i < 0 || _arguments.length <= i ? undefined : _arguments[i];

    if (typeof name === 'string') {
      classNameMap[name] = true;
    } else if (typeof name === 'object') {
      Object.keys(name).forEach(function (key) {
        if (name[key]) {
          classNameMap[key] = true;
        }
      });
    }
  };

  for (var i = 0; i < arguments.length; i++) {
    _loop(i);
  }

  return Object.keys(classNameMap).join(' ');
}
var getDomFromHTMLTemp = function getDomFromHTMLTemp(HTMLTemp) {
  var div = document.createElement('div');
  HTMLTemp = HTMLTemp.trim();
  div.innerHTML = HTMLTemp;
  return div.childNodes;
};

var parseDom = function parseDom(dom, parent) {
  if (parent === void 0) {
    parent = null;
  }

  if (dom instanceof Element) {
    var element = {
      children: [],
      attributes: [],
      parent: parent,
      tagName: dom.tagName.toLocaleLowerCase(),
      type: 'element'
    };
    var attrs = dom.getAttributeNames();
    element.attributes = attrs.map(function (key) {
      var value = dom.getAttribute(key) || '';

      if (key === 'id') {
        element.id = value;
      } else if (key === 'class') {
        element.class = value;
      }

      return {
        name: key,
        value: value
      };
    });
    var childNodes = dom.childNodes;

    if (childNodes && childNodes.length) {
      for (var i = 0; i < childNodes.length; i++) {
        var ast = parseDom(childNodes[i], element);

        if (ast) {
          element.children.push(ast);
        }
      }
    }

    return element;
  } else if (dom instanceof Text) {
    var textContent = dom.textContent || '';
    var text = {
      parent: parent,
      content: textContent,
      type: 'text'
    };
    return text;
  } else if (dom instanceof Comment) {
    return null;
  }

  throw new Error('类型错误');
};

var parse = function parse(template) {
  var doms = getDomFromHTMLTemp(template);
  var nodes = [];

  for (var i = 0; i < doms.length; i++) {
    var ast = parseDom(doms[i]);

    if (ast) {
      nodes.push(ast);
    }
  }

  return nodes;
};

var Parse = /*#__PURE__*/function () {
  function Parse(props) {
    var template = props.template,
        customAttr$1 = props.customAttr,
        customTag$1 = props.customTag;
    this.template = template;
    this.ast = parse(template); // this.list = list;

    this.custom = {
      attrName: customAttr$1 || customAttr,
      tagName: customTag$1 || customTag
    };
  } // getHTML(list?: INoteTextHighlightInfo[]) {
  //   const snapshot = cloneDeep(this.ast);
  //   return getHTML(snapshot, list);
  // }


  var _proto = Parse.prototype;

  _proto.getJSON = function getJSON$1(list) {
    var snapshot = cloneDeep(this.ast);
    return getJSON(snapshot, list);
  };

  return Parse;
}();

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = "[data-wj-custom-split]{cursor:pointer}.text-highlight-wrap{position:relative}";
styleInject(css_248z);

var getContext = function getContext(snapShoot) {
  var nodes = [];

  for (var i = 0; i < snapShoot.length; i++) {
    var node = snapShoot[i];

    if (node.type === 'text') {
      nodes.push(node.content);
    } else if (node.type === 'element') {
      var children = [];

      if (!lodash.isEmpty(node.children)) {
        children = getContext(node.children);
      }

      var props = node.attributes.reduce(function (attrs, attribute) {
        if (attribute.name === 'class') {
          attrs.className = attribute.value;
        } else {
          attrs[attribute.name] = attribute.value;
        }

        return attrs;
      }, {});
      props.key = node.tagName + "-" + i;
      nodes.push(React__default.createElement.apply(React__default, [node.tagName, props].concat(children)));
    } else {
      throw new Error('类型错误');
    }
  }

  return nodes;
};

var Note = function Note(_ref) {
  var template = _ref.template,
      value = _ref.value,
      _ref$tagName = _ref.tagName,
      tagName = _ref$tagName === void 0 ? customTag : _ref$tagName,
      _ref$rowKey = _ref.rowKey,
      rowKey = _ref$rowKey === void 0 ? customRowKey : _ref$rowKey,
      onAdd = _ref.onAdd,
      onUpdate = _ref.onUpdate,
      modes = _ref.modes,
      children = _ref.children;
  var parse = React.useMemo(function () {
    // 用于格式化html文本
    if (template) {
      var div = document.createElement('div');
      div.innerHTML = template;
      template = div.innerHTML;
    }

    return new Parse({
      template: template
    });
  }, []);

  var _useState = React.useState(function () {
    return parse.getJSON(value);
  }),
      snapShoot = _useState[0],
      setSnapShoot = _useState[1];

  var selectedValue = React.useRef(null);
  var action = React.useRef('');
  var noteContainer = React.useRef(null);
  var wrapContainer = React.useRef(null);

  var _useState2 = React.useState({
    selectedValue: selectedValue,
    wrapContainer: wrapContainer,
    action: action
  }),
      contextValue = _useState2[0],
      setContextValue = _useState2[1];

  var handleSelectedText = React.useCallback(function () {
    var _window, _window$getSelection;

    var range = (_window = window) == null ? void 0 : (_window$getSelection = _window.getSelection()) == null ? void 0 : _window$getSelection.getRangeAt(0);
    if (!range) return;
    var text = range.toString();
    var _range$collapsed = range.collapsed,
        collapsed = _range$collapsed === void 0 ? true : _range$collapsed,
        endContainer = range.endContainer,
        startContainer = range.startContainer; // 返回条件 1. 光标起始点相同（即没有选中文本），2. 起点或者终点不在当前容器内

    if (collapsed || !noteContainer.current || !noteContainer.current.contains(startContainer) || !noteContainer.current.contains(endContainer)) return;
    var list = getSelectedInfo({
      range: range,
      noteContainer: noteContainer.current
    });
    var data = {
      list: list,
      text: text
    };
    action.current = 'add';
    selectedValue.current = data;
    onAdd && onAdd(data);
  }, [noteContainer, modes, parse]);
  var handleClick = React.useCallback(function (e) {
    if (!noteContainer.current || !e.target) return;
    var node = e.target;
    var uuids = [];

    while (node) {
      var uuid = node.getAttribute(customSelectedAttr);

      if (uuid) {
        uuids.push(uuid);
        node = node.parentNode;
      } else {
        break;
      }
    }

    if (!uuids.length) return;
    var findData = [];
    uuids.forEach(function (uuid) {
      var data = value == null ? void 0 : value.find(function (item) {
        return item[rowKey] === uuid;
      });

      if (data) {
        findData.push(data);
      }
    });
    action.current = 'update';
    onUpdate && onUpdate(findData);
  }, [noteContainer, value]);
  var handleMouseDown = React.useCallback(function (eventDown) {
    if (!noteContainer.current) return;
    document.addEventListener('mouseup', function (eventUp) {
      handleSelectedText();
    }, {
      once: true
    });
  }, [noteContainer]);
  React.useEffect(function () {
    setContextValue({
      selectedValue: selectedValue,
      action: action,
      wrapContainer: wrapContainer
    });
  }, [selectedValue, action, wrapContainer]);
  React.useEffect(function () {
    // TODO 不造是否有更好的设置值的方式，后续如果有再修改
    var modeClassNames = {};

    if (modes && modes.length) {
      modes.forEach(function (item) {
        if (item.mode) {
          modeClassNames[item.mode] = item.className || '';
        }
      });
    }

    setCustomValue({
      tagName: tagName,
      attrName: customAttr,
      rowKey: rowKey,
      splitAttrName: customSplitAttr,
      selectedAttr: customSelectedAttr,
      modeClassNames: modeClassNames
    });
    setSnapShoot(parse.getJSON(value));
  }, [setSnapShoot, parse, value, modes]);
  return React__default.createElement(context.Provider, {
    value: contextValue
  }, React__default.createElement("div", {
    className: "text-highlight-wrap",
    ref: wrapContainer
  }, React__default.createElement("div", {
    className: "text-highlight",
    id: "text-highlight",
    ref: noteContainer,
    onMouseDown: handleMouseDown,
    onClick: handleClick
  }, getContext(snapShoot)), children));
};

var insideElement = function insideElement(target, wrap) {
  var node = target;

  while (node) {
    if (node === wrap) return true;
    node = node.parentElement;
  }

  return false;
};

var css_248z$1 = ".note-none{display:none}.note-tool-mask{display:block;position:absolute;top:0;left:0;width:100%;height:100%}.note-tool{display:none;margin:0;padding:0;list-style:none;position:absolute;font-weight:300;background:#484848;color:#fff;align-items:center;justify-content:space-around;border-radius:8px}.note-tool.up:before{border-top:6px solid #484848;bottom:-6px}.note-tool.down:before,.note-tool.up:before{content:\"\";position:absolute;border-right:8px solid transparent;border-left:8px solid transparent;width:0;height:0}.note-tool.down:before{border-bottom:6px solid #484848;top:-6px}";
styleInject(css_248z$1);

var ToolBar = function ToolBar(props) {
  var visible = props.visible,
      children = props.children,
      _props$mask = props.mask,
      mask = _props$mask === void 0 ? true : _props$mask,
      _props$autoClosable = props.autoClosable,
      autoClosable = _props$autoClosable === void 0 ? true : _props$autoClosable,
      onCancel = props.onCancel;
  var wrapContext = React.useContext(context);
  var toolContainer = React.useRef(null); // 用于设置弹窗的显隐

  React.useEffect(function () {
    var _window, _window$getSelection;

    var _ref = wrapContext || {},
        wrapContainer = _ref.wrapContainer;

    if (!(wrapContainer != null && wrapContainer.current) || !toolContainer.current) return;
    toolContainer.current.style.display = visible ? 'flex' : 'none';
    if (!visible) return;
    var range; // if (action?.current === 'add') {

    range = (_window = window) == null ? void 0 : (_window$getSelection = _window.getSelection()) == null ? void 0 : _window$getSelection.getRangeAt(0); // TODO 不太确定更新时如何处理，所以直接使用当前点击定位
    // } else if (action?.current === 'update') {
    //   debugger
    // }

    if (!range) return;
    var rangeRect = range.getBoundingClientRect();
    var wrapContainerRect = wrapContainer.current.getBoundingClientRect();
    var toolContainerRect = toolContainer.current.getBoundingClientRect();
    var top;
    var toolClassName;
    var left; // let arrowLeft: number;

    if (rangeRect.top > marginVertical + toolContainerRect.height) {
      toolClassName = 'up';
      top = rangeRect.top - wrapContainerRect.top - marginVertical - toolContainerRect.height;
    } else {
      toolClassName = 'down';
      top = rangeRect.bottom - wrapContainerRect.top + marginVertical;
    }

    var leftPoint = (rangeRect.left + rangeRect.right) / 2 - wrapContainerRect.left;

    if (leftPoint - toolContainerRect.width / 2 < 0) {
      left = 0; // arrowLeft = leftPoint < 6 ? 6 : leftPoint;
    } else if (leftPoint + toolContainerRect.width / 2 > wrapContainerRect.width) {
      left = wrapContainerRect.width - toolContainerRect.width; // arrowLeft = wrapContainerRect.left - leftPoint < 6 ? 6 : leftPoint;
    } else {
      left = (rangeRect.left + rangeRect.right) / 2 - toolContainerRect.width / 2 - wrapContainerRect.left; // arrowLeft = leftPoint;
    }

    toolContainer.current.style.top = top + "px";
    toolContainer.current.style.left = left + "px";
    toolContainer.current.classList.remove('up', 'down');
    toolContainer.current.classList.add(toolClassName);

    var handle = function handle(e) {
      if (!toolContainer.current || typeof onCancel !== 'function' || insideElement(e.target, toolContainer.current)) return;

      if (autoClosable) {
        onCancel();
      }
    }; // TODO 没有想到好的方式去移除弹窗，后续优化


    setTimeout(function () {
      document.addEventListener('click', handle);
    });
    return function () {
      document.removeEventListener('click', handle);
    };
  }, [autoClosable, onCancel, visible, wrapContext]);
  return React__default.createElement(React__default.Fragment, null, React__default.createElement("div", {
    className: classNames('note-none', {
      'note-tool-mask': visible && mask
    })
  }), React__default.createElement("div", {
    className: "note-tool",
    ref: toolContainer
  }, children));
};

var getNode = function getNode(wrap, path) {
  if (!wrap || !path.length) return;
  var node = wrap;
  var index = 0;

  while (node && index < path.length) {
    var _node = node,
        childNodes = _node.childNodes;
    node = childNodes[path[index]];
    index++;
  }

  return node;
}; // TODO 这是一坨代码， 要修改， 太困了先写着
// 这里有BUG


var getRealNodeAndOffset = function getRealNodeAndOffset(node, offset) {
  var _getCustomValue = getCustomValue(),
      attrName = _getCustomValue.attrName; // 当前节点可能为自定义分割节点


  var wrap = node; // 节点为元素，且需要是自定义元素，有挂载自定义属性的元素都为自定义元素

  if (node instanceof Element) {
    if (node.getAttribute(attrName)) {
      var _node$textContent;

      var textOffset = 0;
      var index = 0;
      node = wrap.childNodes[index];

      outer: while (node && textOffset < offset) {
        // 选中开始节点 - 选中的结束节点 之间的文本节点都为选中文本
        // 只标记文本节点
        if (node.nodeType === 3) {
          var _node2, _node2$textContent;

          textOffset += ((_node2 = node) == null ? void 0 : (_node2$textContent = _node2.textContent) == null ? void 0 : _node2$textContent.length) || 0;
          if (textOffset >= offset) break;
        }

        if (node.childNodes.length) {
          node = node.childNodes[0];
        } else if (node.nextSibling) {
          node = node.nextSibling;
        } else {
          do {
            var _node3;

            node = (_node3 = node) == null ? void 0 : _node3.parentNode;

            if (wrap === node) {
              break outer;
            }
          } while (!node.nextSibling);

          node = node.nextSibling;
        }
      }

      return {
        node: node,
        offset: (((_node$textContent = node.textContent) == null ? void 0 : _node$textContent.length) || 0) - (textOffset - offset)
      };
    }

    throw new Error('元素类型错误');
  }

  return {
    node: node,
    offset: offset
  };
};

var selectRange = function selectRange(data) {
  var _document;

  var wrap = document.querySelector('#text-highlight');
  var getSelection = window.getSelection();
  if (!((_document = document) != null && _document.createRange) || !wrap || !getSelection) return;
  var range = new Range();
  var startNode = getNode(wrap, data.list[0].level);
  var startOffset = data.list[0].start;
  var endNode = getNode(wrap, data.list[data.list.length - 1].level);
  var endOffset = data.list[data.list.length - 1].end;
  if (!startNode || !endNode) return;
  var start = getRealNodeAndOffset(startNode, startOffset);
  var end = getRealNodeAndOffset(endNode, endOffset);
  range.setStart(start.node, start.offset);
  range.setEnd(end.node, end.offset);
  if (getSelection.rangeCount > 0) getSelection.removeAllRanges();
  getSelection.addRange(range);
};

exports.ToolBar = ToolBar;
exports.default = Note;
exports.setSelectRange = selectRange;
//# sourceMappingURL=react-web-highlighter.cjs.development.js.map
